# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from typing import Any, NamedTuple

try:
    from airflow.sdk import BaseHook
except ImportError:
    from airflow.hooks.base import BaseHook  # type: ignore[attr-defined,no-redef]


class AccessKeyCredentials(NamedTuple):
    """
    A NamedTuple to store Alibaba Cloud Access Key credentials.

    :param access_key_id: The Access Key ID for Alibaba Cloud authentication.
    :param access_key_secret: The Access Key Secret for Alibaba Cloud authentication.
    """

    access_key_id: str
    access_key_secret: str


class AlibabaBaseHook(BaseHook):
    """
    A base hook for Alibaba Cloud-related hooks.

    This hook provides a common interface for authenticating using Alibaba Cloud credentials.

    Supports Access Key-based authentication.

    :param alibaba_cloud_conn_id: The connection ID to use when fetching connection info.
    """

    conn_name_attr = "alibabacloud_conn_id"
    default_conn_name = "alibabacloud_default"
    conn_type = "alibaba_cloud"
    hook_name = "Alibaba Cloud"

    def __init__(
        self,
        alibabacloud_conn_id: str = "alibabacloud_default",
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.alibaba_cloud_conn_id = alibabacloud_conn_id
        self.extras: dict = self.get_connection(self.alibaba_cloud_conn_id).extra_dejson

    @classmethod
    def get_connection_form_widgets(cls) -> dict[str, Any]:
        """Return connection widgets to add to connection form."""
        from flask_appbuilder.fieldwidgets import BS3PasswordFieldWidget
        from flask_babel import lazy_gettext
        from wtforms import PasswordField

        return {
            "access_key_id": PasswordField(lazy_gettext("Access Key ID"), widget=BS3PasswordFieldWidget()),
            "access_key_secret": PasswordField(
                lazy_gettext("Access Key Secret"), widget=BS3PasswordFieldWidget()
            ),
        }

    @classmethod
    def get_ui_field_behaviour(cls) -> dict[str, Any]:
        """Return custom field behaviour."""
        return super().get_ui_field_behaviour()

    def _get_field(self, field_name: str, default: Any = None) -> Any:
        """Fetch a field from extras, and returns it."""
        value = self.extras.get(field_name)
        return value if value is not None else default

    def get_access_key_credential(self) -> AccessKeyCredentials:
        """
        Fetch Access Key Credential for authentication.

        :return: AccessKeyCredentials object containing access_key_id and access_key_secret.
        """
        access_key_id = self._get_field("access_key_id", None)
        access_key_secret = self._get_field("access_key_secret", None)
        if not access_key_id:
            raise ValueError("No access_key_id is specified.")

        if not access_key_secret:
            raise ValueError("No access_key_secret is specified.")

        return AccessKeyCredentials(access_key_id, access_key_secret)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from typing import TYPE_CHECKING

from airflow.providers.alibaba.version_compat import AIRFLOW_V_3_0_PLUS

if TYPE_CHECKING:
    from airflow.models import BaseOperator
    from airflow.models.taskinstancekey import TaskInstanceKey
    from airflow.utils.context import Context

if AIRFLOW_V_3_0_PLUS:
    from airflow.sdk import BaseOperatorLink
    from airflow.sdk.execution_time.xcom import XCom
else:
    from airflow.models.baseoperatorlink import BaseOperatorLink  # type: ignore[no-redef]
    from airflow.models.xcom import XCom  # type: ignore[no-redef]


class MaxComputeLogViewLink(BaseOperatorLink):
    """Helper class for constructing MaxCompute Log View Link."""

    name = "MaxCompute Log View"
    key = "maxcompute_log_view"

    def get_link(
        self,
        operator: BaseOperator,
        *,
        ti_key: TaskInstanceKey,
    ) -> str:
        url = XCom.get_value(key=self.key, ti_key=ti_key)
        if not url:
            return ""

        return url

    @staticmethod
    def persist(
        context: Context,
        log_view_url: str,
    ):
        """
        Persist the log view URL to XCom for later retrieval.

        :param context: The context of the task instance.
        :param log_view_url: The log view URL to persist.
        """
        context["task_instance"].xcom_push(
            key=MaxComputeLogViewLink.key,
            value=log_view_url,
        )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

import pytest

from airflow.exceptions import AirflowException
from airflow.providers.alibaba.cloud.operators.analyticdb_spark import (
    AnalyticDBSparkBaseOperator,
    AnalyticDBSparkBatchOperator,
    AnalyticDBSparkSQLOperator,
)

ADB_SPARK_OPERATOR_STRING = "airflow.providers.alibaba.cloud.operators.analyticdb_spark.{}"

MOCK_FILE = "oss://test.py"
MOCK_CLUSTER_ID = "mock_cluster_id"
MOCK_RG_NAME = "mock_rg_name"
MOCK_ADB_SPARK_CONN_ID = "mock_adb_spark_conn_id"
MOCK_REGION = "mock_region"
MOCK_TASK_ID = "mock_task_id"
MOCK_APP_ID = "mock_app_id"
MOCK_SQL = "SELECT 1;"


class TestAnalyticDBSparkBaseOperator:
    def setup_method(self):
        self.operator = AnalyticDBSparkBaseOperator(
            adb_spark_conn_id=MOCK_ADB_SPARK_CONN_ID,
            region=MOCK_REGION,
            task_id=MOCK_TASK_ID,
        )

    @mock.patch(ADB_SPARK_OPERATOR_STRING.format("AnalyticDBSparkHook"))
    def test_get_hook(self, mock_hook):
        """Test get_hook function works as expected."""
        self.operator.hook
        mock_hook.assert_called_once_with(adb_spark_conn_id=MOCK_ADB_SPARK_CONN_ID, region=MOCK_REGION)

    @mock.patch(ADB_SPARK_OPERATOR_STRING.format("AnalyticDBSparkBaseOperator.hook"))
    def test_poll_for_termination(self, mock_hook):
        """Test poll_for_termination works as expected with COMPLETED application."""
        # Given
        mock_hook.get_spark_state.return_value = "COMPLETED"

        # When
        self.operator.poll_for_termination(MOCK_APP_ID)

    @mock.patch(ADB_SPARK_OPERATOR_STRING.format("AnalyticDBSparkBaseOperator.hook"))
    def test_poll_for_termination_with_exception(self, mock_hook):
        """Test poll_for_termination raises AirflowException with FATAL application."""
        # Given
        mock_hook.get_spark_state.return_value = "FATAL"

        # When
        with pytest.raises(AirflowException, match="Application mock_app_id did not succeed"):
            self.operator.poll_for_termination(MOCK_APP_ID)


class TestAnalyticDBSparkBatchOperator:
    @mock.patch(ADB_SPARK_OPERATOR_STRING.format("AnalyticDBSparkHook"))
    def test_execute(self, mock_hook):
        """Test submit AnalyticDB Spark Batch Application works as expected."""
        operator = AnalyticDBSparkBatchOperator(
            file=MOCK_FILE,
            cluster_id=MOCK_CLUSTER_ID,
            rg_name=MOCK_RG_NAME,
            adb_spark_conn_id=MOCK_ADB_SPARK_CONN_ID,
            region=MOCK_REGION,
            task_id=MOCK_TASK_ID,
        )

        operator.execute(None)

        mock_hook.assert_called_once_with(adb_spark_conn_id=MOCK_ADB_SPARK_CONN_ID, region=MOCK_REGION)
        mock_hook.return_value.submit_spark_app.assert_called_once_with(
            cluster_id=MOCK_CLUSTER_ID,
            rg_name=MOCK_RG_NAME,
            file=MOCK_FILE,
            class_name=None,
            args=None,
            conf=None,
            jars=None,
            py_files=None,
            files=None,
            driver_resource_spec=None,
            executor_resource_spec=None,
            num_executors=None,
            archives=None,
            name=None,
        )

    @mock.patch(ADB_SPARK_OPERATOR_STRING.format("AnalyticDBSparkBaseOperator.hook"))
    def test_execute_with_exception(self, mock_hook):
        """Test submit AnalyticDB Spark Batch Application raises ValueError with invalid parameter."""
        # Given
        mock_hook.submit_spark_app.side_effect = ValueError("List of strings expected")

        # When
        operator = AnalyticDBSparkBatchOperator(
            file=MOCK_FILE,
            args=(True, False),
            cluster_id=MOCK_CLUSTER_ID,
            rg_name=MOCK_RG_NAME,
            adb_spark_conn_id=MOCK_ADB_SPARK_CONN_ID,
            region=MOCK_REGION,
            task_id=MOCK_TASK_ID,
        )

        with pytest.raises(ValueError, match="List of strings expected"):
            operator.execute(None)


class TestAnalyticDBSparklSQLOperator:
    @mock.patch(ADB_SPARK_OPERATOR_STRING.format("AnalyticDBSparkHook"))
    def test_execute(self, mock_hook):
        """Test submit AnalyticDB Spark SQL Application works as expected."""
        operator = AnalyticDBSparkSQLOperator(
            sql=MOCK_SQL,
            cluster_id=MOCK_CLUSTER_ID,
            rg_name=MOCK_RG_NAME,
            adb_spark_conn_id=MOCK_ADB_SPARK_CONN_ID,
            region=MOCK_REGION,
            task_id=MOCK_TASK_ID,
        )

        operator.execute(None)

        mock_hook.assert_called_once_with(adb_spark_conn_id=MOCK_ADB_SPARK_CONN_ID, region=MOCK_REGION)
        mock_hook.return_value.submit_spark_sql.assert_called_once_with(
            cluster_id=MOCK_CLUSTER_ID,
            rg_name=MOCK_RG_NAME,
            sql=MOCK_SQL,
            conf=None,
            driver_resource_spec=None,
            executor_resource_spec=None,
            num_executors=None,
            name=None,
        )

    @mock.patch(ADB_SPARK_OPERATOR_STRING.format("AnalyticDBSparkBaseOperator.hook"))
    def test_execute_with_exception(self, mock_hook):
        """Test submit AnalyticDB Spark SQL Application raises ValueError with invalid parameter."""
        # Given
        mock_hook.submit_spark_sql.side_effect = ValueError("List of strings expected")

        # When
        operator = AnalyticDBSparkSQLOperator(
            sql=MOCK_SQL,
            conf={"spark.eventLog.enabled": True},
            cluster_id=MOCK_CLUSTER_ID,
            rg_name=MOCK_RG_NAME,
            adb_spark_conn_id=MOCK_ADB_SPARK_CONN_ID,
            region=MOCK_REGION,
            task_id=MOCK_TASK_ID,
        )

        with pytest.raises(ValueError, match="List of strings expected"):
            operator.execute(None)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

from airflow.providers.alibaba.cloud.hooks.maxcompute import MaxComputeHook

MAXCOMPUTE_HOOK_MODULE = "airflow.providers.alibaba.cloud.hooks.maxcompute.MaxComputeHook.{}"
MOCK_MAXCOMPUTE_CONN_ID = "mock_id"
MOCK_MAXCOMPUTE_PROJECT = "mock_project"
MOCK_MAXCOMPUTE_ENDPOINT = "mock_endpoint"


class TestMaxComputeHook:
    def setup_method(self):
        with mock.patch(
            MAXCOMPUTE_HOOK_MODULE.format("get_connection"),
        ) as mock_get_connection:
            mock_conn = mock.MagicMock()
            mock_conn.extra_dejson = {
                "access_key_id": "mock_access_key_id",
                "access_key_secret": "mock_access_key_secret",
                "project": MOCK_MAXCOMPUTE_PROJECT,
                "endpoint": MOCK_MAXCOMPUTE_ENDPOINT,
            }

            mock_get_connection.return_value = mock_conn
            self.hook = MaxComputeHook(maxcompute_conn_id=MOCK_MAXCOMPUTE_CONN_ID)

    @mock.patch(MAXCOMPUTE_HOOK_MODULE.format("get_client"))
    def test_run_sql(self, mock_get_client):
        mock_instance = mock.MagicMock()
        mock_client = mock.MagicMock()
        mock_client.run_sql.return_value = mock_instance
        mock_get_client.return_value = mock_client

        sql = "SELECT 1"
        priority = 1
        running_cluster = "mock_running_cluster"
        hints = {"hint_key": "hint_value"}
        aliases = {"alias_key": "alias_value"}
        default_schema = "mock_default_schema"
        quota_name = "mock_quota_name"

        instance = self.hook.run_sql(
            sql=sql,
            priority=priority,
            running_cluster=running_cluster,
            hints=hints,
            aliases=aliases,
            default_schema=default_schema,
            quota_name=quota_name,
        )

        assert instance == mock_instance

        assert mock_client.run_sql.asssert_called_once_with(
            sql=sql,
            priority=priority,
            running_cluster=running_cluster,
            hints=hints,
            aliases=aliases,
            default_schema=default_schema,
            quota_name=quota_name,
        )

    @mock.patch(MAXCOMPUTE_HOOK_MODULE.format("get_client"))
    def test_get_instance(self, mock_get_client):
        mock_client = mock.MagicMock()
        mock_client.exist_instance.return_value = True
        mock_instance = mock.MagicMock()
        mock_client.get_instance.return_value = mock_instance
        mock_get_client.return_value = mock_client
        instance_id = "mock_instance_id"

        instance = self.hook.get_instance(
            instance_id=instance_id,
            project=MOCK_MAXCOMPUTE_PROJECT,
            endpoint=MOCK_MAXCOMPUTE_ENDPOINT,
        )

        mock_client.get_instance.assert_called_once_with(
            id_=instance_id,
            project=MOCK_MAXCOMPUTE_PROJECT,
        )

        assert instance == mock_instance

    @mock.patch(MAXCOMPUTE_HOOK_MODULE.format("get_client"))
    def test_stop_instance_success(self, mock_get_client):
        mock_client = mock.MagicMock()
        mock_get_client.return_value = mock_client
        instance_id = "mock_instance_id"

        self.hook.stop_instance(
            instance_id=instance_id,
            project=MOCK_MAXCOMPUTE_PROJECT,
            endpoint=MOCK_MAXCOMPUTE_ENDPOINT,
        )

        mock_client.stop_instance.assert_called_once()



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import contextlib
import os
import shutil
from functools import cached_property
from pathlib import Path
from typing import TYPE_CHECKING

import attrs

from airflow.configuration import conf
from airflow.providers.alibaba.cloud.hooks.oss import OSSHook
from airflow.utils.log.file_task_handler import FileTaskHandler
from airflow.utils.log.logging_mixin import LoggingMixin

if TYPE_CHECKING:
    from airflow.sdk.types import RuntimeTaskInstanceProtocol as RuntimeTI
    from airflow.utils.log.file_task_handler import LogMessages, LogSourceInfo


@attrs.define(kw_only=True)
class OSSRemoteLogIO(LoggingMixin):  # noqa: D101
    base_log_folder: Path = attrs.field(converter=Path)
    remote_base: str = ""
    delete_local_copy: bool = True

    processors = ()

    def upload(self, path: os.PathLike | str, ti: RuntimeTI):
        """Upload the given log path to the remote storage."""
        path = Path(path)
        if path.is_absolute():
            local_loc = path
            remote_loc = os.path.join(self.remote_base, path.relative_to(self.base_log_folder))
        else:
            local_loc = self.base_log_folder.joinpath(path)
            remote_loc = os.path.join(self.remote_base, path)

        if local_loc.is_file():
            # read log and remove old logs to get just the latest additions
            log = local_loc.read_text()
            has_uploaded = self.oss_write(log, remote_loc)
            if has_uploaded and self.delete_local_copy:
                shutil.rmtree(os.path.dirname(local_loc))

    @cached_property
    def base_folder(self):
        (_, base_folder) = OSSHook.parse_oss_url(self.remote_base)
        return base_folder

    @cached_property
    def bucket_name(self):
        (bucket_name, _) = OSSHook.parse_oss_url(self.remote_base)
        return bucket_name

    @cached_property
    def hook(self):
        remote_conn_id = conf.get("logging", "REMOTE_LOG_CONN_ID")
        self.log.info("remote_conn_id: %s", remote_conn_id)
        try:
            return OSSHook(oss_conn_id=remote_conn_id)
        except Exception as e:
            self.log.exception(e)
            self.log.error(
                'Could not create an OSSHook with connection id "%s". '
                "Please make sure that airflow[oss] is installed and "
                "the OSS connection exists.",
                remote_conn_id,
            )

    def read(self, relative_path, ti: RuntimeTI) -> tuple[LogSourceInfo, LogMessages | None]:
        logs: list[str] = []
        messages = [relative_path]

        if self.oss_log_exists(relative_path):
            logs.append(self.oss_read(relative_path, return_error=True))
            return messages, logs
        return messages, None

    def oss_log_exists(self, remote_log_location):
        """
        Check if remote_log_location exists in remote storage.

        :param remote_log_location: log's location in remote storage
        :return: True if location exists else False
        """
        oss_remote_log_location = f"{self.base_folder}/{remote_log_location}"
        with contextlib.suppress(Exception):
            return self.hook.key_exist(self.bucket_name, oss_remote_log_location)
        return False

    def oss_read(self, remote_log_location, return_error=False):
        """
        Return the log at the remote_log_location or '' if no logs are found or there is an error.

        :param remote_log_location: the log's location in remote storage
        :param return_error: if True, returns a string error message if an
            error occurs. Otherwise, returns '' when an error occurs.
        """
        oss_remote_log_location = f"{self.base_folder}/{remote_log_location}"
        try:
            self.log.info("read remote log: %s", oss_remote_log_location)
            return self.hook.read_key(self.bucket_name, oss_remote_log_location)
        except Exception:
            msg = f"Could not read logs from {oss_remote_log_location}"
            self.log.exception(msg)
            # return error if needed
            if return_error:
                return msg

    def oss_write(self, log, remote_log_location, append=True) -> bool:
        """
        Write the log to remote_log_location and return `True`; fails silently and returns `False` on error.

        :param log: the log to write to the remote_log_location
        :param remote_log_location: the log's location in remote storage
        :param append: if False, any existing log file is overwritten. If True,
            the new log is appended to any existing logs.
        :return: whether the log is successfully written to remote location or not.
        """
        oss_remote_log_location = f"{self.base_folder}/{remote_log_location}"
        pos = 0
        if append and self.oss_log_exists(remote_log_location):
            head = self.hook.head_key(self.bucket_name, oss_remote_log_location)
            pos = head.content_length
        self.log.info("log write pos is: %s", pos)
        try:
            self.log.info("writing remote log: %s", oss_remote_log_location)
            self.hook.append_string(self.bucket_name, log, oss_remote_log_location, pos)
        except Exception:
            self.log.exception(
                "Could not write logs to %s, log write pos is: %s, Append is %s",
                oss_remote_log_location,
                pos,
                append,
            )
            return False
        return True


class OSSTaskHandler(FileTaskHandler, LoggingMixin):
    """
    OSSTaskHandler is a python log handler that handles and reads task instance logs.

    Extends airflow FileTaskHandler and uploads to and reads from OSS remote storage.
    """

    def __init__(self, base_log_folder, oss_log_folder, **kwargs):
        self.log.info("Using oss_task_handler for remote logging...")
        super().__init__(base_log_folder)
        self.log_relative_path = ""
        self._hook = None
        self.closed = False
        self.upload_on_close = True
        self.delete_local_copy = kwargs.get(
            "delete_local_copy", conf.getboolean("logging", "delete_local_logs")
        )

        self.io = OSSRemoteLogIO(
            remote_base=oss_log_folder,
            base_log_folder=base_log_folder,
            delete_local_copy=kwargs.get(
                "delete_local_copy", conf.getboolean("logging", "delete_local_logs")
            ),
        )

    def set_context(self, ti):
        """Set the context of the handler."""
        super().set_context(ti)
        # Local location and remote location is needed to open and
        # upload local log file to OSS remote storage.
        self.log_relative_path = self._render_filename(ti, ti.try_number)
        self.upload_on_close = not ti.raw

        self.ti = ti
        # Clear the file first so that duplicate data is not uploaded
        # when reusing the same path (e.g. with rescheduled sensors)
        if self.upload_on_close:
            with open(self.handler.baseFilename, "w"):
                pass

    def close(self):
        """Close and upload local log file to remote storage OSS."""
        # When application exit, system shuts down all handlers by
        # calling close method. Here we check if logger is already
        # closed to prevent uploading the log to remote storage multiple
        # times when `logging.shutdown` is called.
        if self.closed:
            return

        super().close()

        if not self.upload_on_close:
            return

        if hasattr(self, "ti"):
            self.io.upload(self.log_relative_path, self.ti)

        # Mark closed so we don't double write if close is called twice
        self.closed = True

    def _read(self, ti, try_number, metadata=None):
        """
        Read logs of given task instance and try_number from OSS remote storage.

        If failed, read the log from task instance host machine.

        :param ti: task instance object
        :param try_number: task instance try_number to read logs from
        :param metadata: log metadata,
                         can be used for steaming log reading and auto-tailing.
        """
        # Explicitly getting log relative path is necessary as the given
        # task instance might be different from task instance passed in
        # set_context method.
        log_relative_path = self._render_filename(ti, try_number)
        remote_loc = log_relative_path

        if not self.oss_log_exists(remote_loc):
            return super()._read(ti, try_number, metadata)
        # If OSS remote file exists, we do not fetch logs from task instance
        # local machine even if there are errors reading remote logs, as
        # returned remote_log will contain error messages.
        remote_log = self.oss_read(remote_loc, return_error=True)
        log = f"*** Reading remote log from {remote_loc}.\n{remote_log}\n"
        return log, {"end_of_log": True}



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import time
from collections.abc import Sequence
from functools import cached_property
from typing import TYPE_CHECKING, Any

from airflow.exceptions import AirflowException
from airflow.providers.alibaba.cloud.hooks.analyticdb_spark import AnalyticDBSparkHook, AppState
from airflow.providers.alibaba.version_compat import AIRFLOW_V_3_0_PLUS

if AIRFLOW_V_3_0_PLUS:
    from airflow.sdk import BaseOperator
else:
    from airflow.models import BaseOperator

if TYPE_CHECKING:
    from airflow.utils.context import Context


class AnalyticDBSparkBaseOperator(BaseOperator):
    """Abstract base class that defines how users develop AnalyticDB Spark."""

    def __init__(
        self,
        *,
        adb_spark_conn_id: str = "adb_spark_default",
        region: str | None = None,
        polling_interval: int = 0,
        **kwargs: Any,
    ) -> None:
        super().__init__(**kwargs)

        self.app_id: str | None = None
        self.polling_interval = polling_interval

        self._adb_spark_conn_id = adb_spark_conn_id
        self._region = region

    @cached_property
    def hook(self) -> AnalyticDBSparkHook:
        """Get valid hook."""
        return AnalyticDBSparkHook(adb_spark_conn_id=self._adb_spark_conn_id, region=self._region)

    def execute(self, context: Context) -> Any: ...

    def monitor_application(self):
        self.log.info("Monitoring application with %s", self.app_id)

        if self.polling_interval > 0:
            self.poll_for_termination(self.app_id)

    def poll_for_termination(self, app_id: str) -> None:
        """
        Pool for spark application termination.

        :param app_id: id of the spark application to monitor
        """
        state = self.hook.get_spark_state(app_id)
        while AppState(state) not in AnalyticDBSparkHook.TERMINAL_STATES:
            self.log.debug("Application with id %s is in state: %s", app_id, state)
            time.sleep(self.polling_interval)
            state = self.hook.get_spark_state(app_id)
        self.log.info("Application with id %s terminated with state: %s", app_id, state)
        self.log.info(
            "Web ui address is %s for application with id %s",
            self.hook.get_spark_web_ui_address(app_id),
            app_id,
        )
        self.log.info(self.hook.get_spark_log(app_id))
        if AppState(state) != AppState.COMPLETED:
            raise AirflowException(f"Application {app_id} did not succeed")

    def on_kill(self) -> None:
        self.kill()

    def kill(self) -> None:
        """Delete the specified application."""
        if self.app_id is not None:
            self.hook.kill_spark_app(self.app_id)


class AnalyticDBSparkSQLOperator(AnalyticDBSparkBaseOperator):
    """
    Submits a Spark SQL application to the underlying cluster; wraps the AnalyticDB Spark REST API.

    :param sql: The SQL query to execute.
    :param conf: Spark configuration properties.
    :param driver_resource_spec: The resource specifications of the Spark driver.
    :param executor_resource_spec: The resource specifications of each Spark executor.
    :param num_executors: number of executors to launch for this application.
    :param name: name of this application.
    :param cluster_id: The cluster ID of AnalyticDB MySQL 3.0 Data Lakehouse.
    :param rg_name: The name of resource group in AnalyticDB MySQL 3.0 Data Lakehouse cluster.
    """

    template_fields: Sequence[str] = ("spark_params",)
    template_fields_renderers = {"spark_params": "json"}

    def __init__(
        self,
        *,
        sql: str,
        conf: dict[Any, Any] | None = None,
        driver_resource_spec: str | None = None,
        executor_resource_spec: str | None = None,
        num_executors: int | str | None = None,
        name: str | None = None,
        cluster_id: str,
        rg_name: str,
        **kwargs: Any,
    ) -> None:
        super().__init__(**kwargs)

        spark_params = {
            "sql": sql,
            "conf": conf,
            "driver_resource_spec": driver_resource_spec,
            "executor_resource_spec": executor_resource_spec,
            "num_executors": num_executors,
            "name": name,
        }
        self.spark_params = spark_params
        self._cluster_id = cluster_id
        self._rg_name = rg_name

    def execute(self, context: Context) -> Any:
        submit_response = self.hook.submit_spark_sql(
            cluster_id=self._cluster_id, rg_name=self._rg_name, **self.spark_params
        )
        self.app_id = submit_response.body.data.app_id
        self.monitor_application()
        return self.app_id


class AnalyticDBSparkBatchOperator(AnalyticDBSparkBaseOperator):
    """
    Submits a Spark batch application to the underlying cluster; wraps the AnalyticDB Spark REST API.

    :param file: path of the file containing the application to execute.
    :param class_name: name of the application Java/Spark main class.
    :param args: application command line arguments.
    :param conf: Spark configuration properties.
    :param jars: jars to be used in this application.
    :param py_files: python files to be used in this application.
    :param files: files to be used in this application.
    :param driver_resource_spec: The resource specifications of the Spark driver.
    :param executor_resource_spec: The resource specifications of each Spark executor.
    :param num_executors: number of executors to launch for this application.
    :param archives: archives to be used in this application.
    :param name: name of this application.
    :param cluster_id: The cluster ID of AnalyticDB MySQL 3.0 Data Lakehouse.
    :param rg_name: The name of resource group in AnalyticDB MySQL 3.0 Data Lakehouse cluster.
    """

    template_fields: Sequence[str] = ("spark_params",)
    template_fields_renderers = {"spark_params": "json"}

    def __init__(
        self,
        *,
        file: str,
        class_name: str | None = None,
        args: Sequence[str | int | float] | None = None,
        conf: dict[Any, Any] | None = None,
        jars: Sequence[str] | None = None,
        py_files: Sequence[str] | None = None,
        files: Sequence[str] | None = None,
        driver_resource_spec: str | None = None,
        executor_resource_spec: str | None = None,
        num_executors: int | str | None = None,
        archives: Sequence[str] | None = None,
        name: str | None = None,
        cluster_id: str,
        rg_name: str,
        **kwargs: Any,
    ) -> None:
        super().__init__(**kwargs)

        spark_params = {
            "file": file,
            "class_name": class_name,
            "args": args,
            "conf": conf,
            "jars": jars,
            "py_files": py_files,
            "files": files,
            "driver_resource_spec": driver_resource_spec,
            "executor_resource_spec": executor_resource_spec,
            "num_executors": num_executors,
            "archives": archives,
            "name": name,
        }
        self.spark_params = spark_params
        self._cluster_id = cluster_id
        self._rg_name = rg_name

    def execute(self, context: Context) -> Any:
        submit_response = self.hook.submit_spark_app(
            cluster_id=self._cluster_id, rg_name=self._rg_name, **self.spark_params
        )
        self.app_id = submit_response.body.data.app_id
        self.monitor_application()
        return self.app_id



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from collections.abc import Callable
from functools import wraps
from inspect import signature
from typing import TypeVar, cast

T = TypeVar("T", bound=Callable)


def skip_test_if_no_valid_conn_id(func: T) -> T:
    """
    Function decorator that skip this test function if no valid connection id is specified.
    """
    function_signature = signature(func)

    @wraps(func)
    def wrapper(*args, **kwargs) -> None:
        bound_args = function_signature.bind(*args, **kwargs)
        self = args[0]

        if self.hook is not None:
            return func(*bound_args.args, **bound_args.kwargs)
        return None

    return cast("T", wrapper)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock
from unittest.mock import PropertyMock

import pytest

from airflow.exceptions import AirflowException
from airflow.providers.alibaba.cloud.sensors.oss_key import OSSKeySensor

MODULE_NAME = "airflow.providers.alibaba.cloud.sensors.oss_key"

MOCK_TASK_ID = "test-oss-operator"
MOCK_REGION = "mock_region"
MOCK_BUCKET = "mock_bucket_name"
MOCK_OSS_CONN_ID = "mock_oss_conn_default"
MOCK_KEY = "mock_key"
MOCK_KEYS = ["mock_key1", "mock_key_2", "mock_key3"]
MOCK_CONTENT = "mock_content"

# mypy: disable-error-code="call-overload"


@pytest.fixture
def oss_key_sensor():
    return OSSKeySensor(
        bucket_key=MOCK_KEY,
        oss_conn_id=MOCK_OSS_CONN_ID,
        region=MOCK_REGION,
        bucket_name=MOCK_BUCKET,
        task_id=MOCK_TASK_ID,
    )


class TestOSSKeySensor:
    @mock.patch(f"{MODULE_NAME}.OSSHook")
    def test_get_hook(self, mock_service, oss_key_sensor):
        oss_key_sensor.hook
        mock_service.assert_called_once_with(oss_conn_id=MOCK_OSS_CONN_ID, region=MOCK_REGION)

    @mock.patch(f"{MODULE_NAME}.OSSKeySensor.hook", new_callable=PropertyMock)
    def test_poke_exsiting_key(self, mock_service, oss_key_sensor):
        # Given
        mock_service.return_value.object_exists.return_value = True

        # When
        res = oss_key_sensor.poke(None)

        # Then
        assert res is True
        mock_service.return_value.object_exists.assert_called_once_with(key=MOCK_KEY, bucket_name=MOCK_BUCKET)

    @mock.patch(f"{MODULE_NAME}.OSSKeySensor.hook", new_callable=PropertyMock)
    def test_poke_non_exsiting_key(self, mock_service, oss_key_sensor):
        # Given
        mock_service.return_value.object_exists.return_value = False

        # When
        res = oss_key_sensor.poke(None)

        # Then
        assert res is False
        mock_service.return_value.object_exists.assert_called_once_with(key=MOCK_KEY, bucket_name=MOCK_BUCKET)

    @mock.patch(f"{MODULE_NAME}.OSSKeySensor.hook", new_callable=PropertyMock)
    def test_poke_without_bucket_name(
        self,
        mock_service,
        oss_key_sensor,
    ):
        # Given
        oss_key_sensor.bucket_name = None
        mock_service.return_value.object_exists.return_value = False

        # When, Then
        with pytest.raises(
            AirflowException, match="If key is a relative path from root, please provide a bucket_name"
        ):
            oss_key_sensor.poke(None)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

from airflow.providers.alibaba.cloud.operators.maxcompute import MaxComputeSQLOperator

MAXCOMPUTE_OPERATOR_MODULE = "airflow.providers.alibaba.cloud.operators.maxcompute.{}"
MOCK_TASK_ID = "run_sql"
MOCK_SQL = "SELECT 1"
MOCK_INSTANCE_ID = "mock_instance_id"


class TestMaxComputeSQLOperator:
    @mock.patch(MAXCOMPUTE_OPERATOR_MODULE.format("MaxComputeLogViewLink"))
    @mock.patch(MAXCOMPUTE_OPERATOR_MODULE.format("MaxComputeHook"))
    def test_execute(self, mock_hook, mock_log_view_link):
        instance_mock = mock.MagicMock()
        instance_mock.id = MOCK_INSTANCE_ID
        instance_mock.get_logview_address.return_value = "http://mock_logview_address"

        mock_hook.return_value.run_sql.return_value = instance_mock

        op = MaxComputeSQLOperator(
            task_id=MOCK_TASK_ID,
            sql=MOCK_SQL,
        )

        instance_id = op.execute(context=mock.MagicMock())

        assert instance_id == instance_mock.id

        mock_hook.return_value.run_sql.assert_called_once_with(
            project=op.project,
            sql=op.sql,
            endpoint=op.endpoint,
            priority=op.priority,
            running_cluster=op.running_cluster,
            hints=op.hints,
            aliases=op.aliases,
            default_schema=op.default_schema,
            quota_name=op.quota_name,
        )

        mock_log_view_link.persist.assert_called_once_with(
            context=mock.ANY,
            log_view_url=instance_mock.get_logview_address.return_value,
        )

    @mock.patch(MAXCOMPUTE_OPERATOR_MODULE.format("MaxComputeHook"))
    def test_on_kill(self, mock_hook):
        instance_mock = mock.MagicMock()
        instance_mock.id = MOCK_INSTANCE_ID

        mock_hook.return_value.run_sql.return_value = instance_mock

        op = MaxComputeSQLOperator(
            task_id=MOCK_TASK_ID,
            sql=MOCK_SQL,
            cancel_on_kill=False,
        )
        op.execute(context=mock.MagicMock())

        op.on_kill()
        mock_hook.return_value.cancel_job.assert_not_called()

        op.cancel_on_kill = True
        op.on_kill()
        mock_hook.return_value.stop_instance.assert_called_once_with(
            instance_id=instance_mock.id, project=op.project, endpoint=op.endpoint
        )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import os
from typing import TYPE_CHECKING
from unittest import mock
from unittest.mock import PropertyMock

import pytest

from airflow.providers.alibaba.cloud.log.oss_task_handler import OSSRemoteLogIO, OSSTaskHandler  # noqa: F401
from airflow.utils.state import TaskInstanceState
from airflow.utils.timezone import datetime

from tests_common.test_utils.config import conf_vars
from tests_common.test_utils.db import clear_db_dags, clear_db_runs

if TYPE_CHECKING:
    from tests_common.pytest_plugin import CreateTaskInstance, DagMaker

pytestmark = pytest.mark.db_test

OSS_TASK_HANDLER_STRING = "airflow.providers.alibaba.cloud.log.oss_task_handler.{}"
MOCK_OSS_CONN_ID = "mock_id"
MOCK_BUCKET_NAME = "mock_bucket_name"
MOCK_KEY = "mock_key"
MOCK_KEYS = ["mock_key1", "mock_key2", "mock_key3"]
MOCK_CONTENT = "mock_content"
MOCK_FILE_PATH = "mock_file_path"


class TestOSSTaskHandler:
    def setup_method(self):
        self.base_log_folder = "local/airflow/logs/1.log"
        self.oss_log_folder = f"oss://{MOCK_BUCKET_NAME}/airflow/logs"
        self.oss_task_handler = OSSTaskHandler(self.base_log_folder, self.oss_log_folder)

    @pytest.fixture(autouse=True)
    def task_instance(self, create_task_instance: CreateTaskInstance, dag_maker: DagMaker):
        self.ti = ti = create_task_instance(
            dag_id="dag_for_testing_oss_task_handler",
            task_id="task_for_testing_oss_task_handler",
            logical_date=datetime(2020, 1, 1),
            state=TaskInstanceState.RUNNING,
        )
        ti.try_number = 1
        ti.raw = False
        dag_maker.session.merge(ti)
        dag_maker.session.commit()
        yield
        clear_db_runs()
        clear_db_dags()

    @mock.patch(OSS_TASK_HANDLER_STRING.format("conf.get"))
    @mock.patch(OSS_TASK_HANDLER_STRING.format("OSSHook"))
    def test_hook(self, mock_service, mock_conf_get):
        # Given
        mock_conf_get.return_value = "oss_default"

        # When
        self.oss_task_handler.io.hook

        # Then
        mock_conf_get.assert_called_once_with("logging", "REMOTE_LOG_CONN_ID")
        mock_service.assert_called_once_with(oss_conn_id="oss_default")

    @mock.patch(OSS_TASK_HANDLER_STRING.format("OSSRemoteLogIO.hook"), new_callable=PropertyMock)
    def test_oss_log_exists(self, mock_service):
        self.oss_task_handler.io.oss_log_exists("1.log")
        mock_service.assert_called_once_with()
        mock_service.return_value.key_exist.assert_called_once_with(MOCK_BUCKET_NAME, "airflow/logs/1.log")

    @mock.patch(OSS_TASK_HANDLER_STRING.format("OSSRemoteLogIO.hook"), new_callable=PropertyMock)
    def test_oss_read(self, mock_service):
        self.oss_task_handler.io.oss_read("1.log")
        mock_service.assert_called_once_with()
        mock_service.return_value.read_key(MOCK_BUCKET_NAME, "airflow/logs/1.log")

    @mock.patch(OSS_TASK_HANDLER_STRING.format("OSSRemoteLogIO.oss_log_exists"))
    @mock.patch(OSS_TASK_HANDLER_STRING.format("OSSRemoteLogIO.hook"), new_callable=PropertyMock)
    def test_oss_write_into_remote_existing_file_via_append(self, mock_service, mock_oss_log_exists):
        # Given
        mock_oss_log_exists.return_value = True
        mock_service.return_value.head_key.return_value.content_length = 1

        # When
        self.oss_task_handler.io.oss_write(MOCK_CONTENT, "1.log", append=True)

        # Then
        assert mock_service.call_count == 2
        mock_service.return_value.head_key.assert_called_once_with(MOCK_BUCKET_NAME, "airflow/logs/1.log")
        mock_oss_log_exists.assert_called_once_with("1.log")
        mock_service.return_value.append_string.assert_called_once_with(
            MOCK_BUCKET_NAME, MOCK_CONTENT, "airflow/logs/1.log", 1
        )

    @mock.patch(OSS_TASK_HANDLER_STRING.format("OSSRemoteLogIO.oss_log_exists"))
    @mock.patch(OSS_TASK_HANDLER_STRING.format("OSSRemoteLogIO.hook"), new_callable=PropertyMock)
    def test_oss_write_into_remote_non_existing_file_via_append(self, mock_service, mock_oss_log_exists):
        # Given
        mock_oss_log_exists.return_value = False

        # When
        self.oss_task_handler.io.oss_write(MOCK_CONTENT, "1.log", append=True)

        # Then
        assert mock_service.call_count == 1
        mock_service.return_value.head_key.assert_not_called()
        mock_oss_log_exists.assert_called_once_with("1.log")
        mock_service.return_value.append_string.assert_called_once_with(
            MOCK_BUCKET_NAME, MOCK_CONTENT, "airflow/logs/1.log", 0
        )

    @mock.patch(OSS_TASK_HANDLER_STRING.format("OSSRemoteLogIO.oss_log_exists"))
    @mock.patch(OSS_TASK_HANDLER_STRING.format("OSSRemoteLogIO.hook"), new_callable=PropertyMock)
    def test_oss_write_into_remote_existing_file_not_via_append(self, mock_service, mock_oss_log_exists):
        # Given
        mock_oss_log_exists.return_value = True

        # When
        self.oss_task_handler.io.oss_write(MOCK_CONTENT, "1.log", append=False)

        # Then
        assert mock_service.call_count == 1
        mock_service.return_value.head_key.assert_not_called()
        mock_oss_log_exists.assert_not_called()
        mock_service.return_value.append_string.assert_called_once_with(
            MOCK_BUCKET_NAME, MOCK_CONTENT, "airflow/logs/1.log", 0
        )

    @mock.patch(OSS_TASK_HANDLER_STRING.format("OSSRemoteLogIO.oss_log_exists"))
    @mock.patch(OSS_TASK_HANDLER_STRING.format("OSSRemoteLogIO.hook"), new_callable=PropertyMock)
    def test_oss_write_into_remote_non_existing_file_not_via_append(self, mock_service, mock_oss_log_exists):
        # Given
        mock_oss_log_exists.return_value = False

        # When
        self.oss_task_handler.io.oss_write(MOCK_CONTENT, "1.log", append=False)

        # Then
        assert mock_service.call_count == 1
        mock_service.return_value.head_key.assert_not_called()
        mock_oss_log_exists.assert_not_called()
        mock_service.return_value.append_string.assert_called_once_with(
            MOCK_BUCKET_NAME, MOCK_CONTENT, "airflow/logs/1.log", 0
        )

    @pytest.mark.parametrize(
        "delete_local_copy, expected_existence_of_local_copy",
        [(True, False), (False, True)],
    )
    @mock.patch(OSS_TASK_HANDLER_STRING.format("OSSRemoteLogIO.hook"), new_callable=PropertyMock)
    def test_close_with_delete_local_copy_conf(
        self,
        mock_service,
        tmp_path_factory,
        delete_local_copy,
        expected_existence_of_local_copy,
    ):
        local_log_path = str(tmp_path_factory.mktemp("local-oss-log-location"))
        with conf_vars({("logging", "delete_local_logs"): str(delete_local_copy)}):
            handler = OSSTaskHandler(local_log_path, self.oss_log_folder)

        handler.log.info("test")
        handler.set_context(self.ti)
        assert handler.upload_on_close

        handler.close()
        assert os.path.exists(handler.handler.baseFilename) == expected_existence_of_local_copy

    def test_filename_template_for_backward_compatibility(self):
        # filename_template arg support for running the latest provider on airflow 2
        OSSTaskHandler(self.base_log_folder, self.oss_log_folder, filename_template=None)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# NOTE! THIS FILE IS AUTOMATICALLY GENERATED AND WILL BE OVERWRITTEN!
#
# IF YOU WANT TO MODIFY THIS FILE, YOU SHOULD MODIFY THE TEMPLATE
# `get_provider_info_TEMPLATE.py.jinja2` IN the `dev/breeze/src/airflow_breeze/templates` DIRECTORY


def get_provider_info():
    return {
        "package-name": "apache-airflow-providers-alibaba",
        "name": "Alibaba",
        "description": "Alibaba Cloud integration (including `Alibaba Cloud <https://www.alibabacloud.com/>`__).\n",
        "integrations": [
            {
                "integration-name": "Alibaba Cloud OSS",
                "external-doc-url": "https://www.alibabacloud.com/help/product/31815.htm",
                "logo": "/docs/integration-logos/alibabacloud-oss.png",
                "how-to-guide": ["/docs/apache-airflow-providers-alibaba/operators/oss.rst"],
                "tags": ["alibaba"],
            },
            {
                "integration-name": "Alibaba Cloud AnalyticDB Spark",
                "external-doc-url": "https://www.alibabacloud.com/help/en/analyticdb-for-mysql/latest/spark-developerment",
                "how-to-guide": ["/docs/apache-airflow-providers-alibaba/operators/analyticdb_spark.rst"],
                "tags": ["alibaba"],
            },
        ],
        "operators": [
            {
                "integration-name": "Alibaba Cloud OSS",
                "python-modules": ["airflow.providers.alibaba.cloud.operators.oss"],
            },
            {
                "integration-name": "Alibaba Cloud AnalyticDB Spark",
                "python-modules": ["airflow.providers.alibaba.cloud.operators.analyticdb_spark"],
            },
            {
                "integration-name": "Alibaba Cloud MaxCompute",
                "python-modules": ["airflow.providers.alibaba.cloud.operators.maxcompute"],
            },
        ],
        "sensors": [
            {
                "integration-name": "Alibaba Cloud OSS",
                "python-modules": ["airflow.providers.alibaba.cloud.sensors.oss_key"],
            },
            {
                "integration-name": "Alibaba Cloud AnalyticDB Spark",
                "python-modules": ["airflow.providers.alibaba.cloud.sensors.analyticdb_spark"],
            },
        ],
        "hooks": [
            {
                "integration-name": "Alibaba Cloud OSS",
                "python-modules": ["airflow.providers.alibaba.cloud.hooks.oss"],
            },
            {
                "integration-name": "Alibaba Cloud AnalyticDB Spark",
                "python-modules": ["airflow.providers.alibaba.cloud.hooks.analyticdb_spark"],
            },
            {
                "integration-name": "Alibaba Cloud",
                "python-modules": ["airflow.providers.alibaba.cloud.hooks.base_alibaba"],
            },
            {
                "integration-name": "Alibaba Cloud MaxCompute",
                "python-modules": ["airflow.providers.alibaba.cloud.hooks.maxcompute"],
            },
        ],
        "connection-types": [
            {
                "hook-class-name": "airflow.providers.alibaba.cloud.hooks.oss.OSSHook",
                "connection-type": "oss",
            },
            {
                "hook-class-name": "airflow.providers.alibaba.cloud.hooks.analyticdb_spark.AnalyticDBSparkHook",
                "connection-type": "adb_spark",
            },
            {
                "hook-class-name": "airflow.providers.alibaba.cloud.hooks.base_alibaba.AlibabaBaseHook",
                "connection-type": "alibaba_cloud",
            },
            {
                "hook-class-name": "airflow.providers.alibaba.cloud.hooks.maxcompute.MaxComputeHook",
                "connection-type": "maxcompute",
            },
        ],
        "logging": ["airflow.providers.alibaba.cloud.log.oss_task_handler.OSSTaskHandler"],
        "extra-links": ["airflow.providers.alibaba.cloud.links.maxcompute.MaxComputeLogViewLink"],
    }



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import os
from datetime import datetime

from airflow.models.dag import DAG
from airflow.providers.alibaba.cloud.operators.oss import (
    OSSDeleteBatchObjectOperator,
    OSSDeleteObjectOperator,
    OSSDownloadObjectOperator,
    OSSUploadObjectOperator,
)

# Ignore missing args provided by default_args
# mypy: disable-error-code="call-arg"

ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID")
DAG_ID = "oss_object_dag"
REGION = os.environ.get("REGION", "default_region")

with DAG(
    dag_id=DAG_ID,
    start_date=datetime(2021, 1, 1),
    schedule=None,
    default_args={"bucket_name": "your bucket", "region": "your region"},
    max_active_runs=1,
    tags=["example"],
    catchup=False,
) as dag:
    create_object = OSSUploadObjectOperator(
        file="your local file",
        key="your oss key",
        task_id="task1",
        region=REGION,
    )

    download_object = OSSDownloadObjectOperator(
        file="your local file",
        key="your oss key",
        task_id="task2",
        region=REGION,
    )

    delete_object = OSSDeleteObjectOperator(
        key="your oss key",
        task_id="task3",
        region=REGION,
    )

    delete_batch_object = OSSDeleteBatchObjectOperator(
        keys=["obj1", "obj2", "obj3"],
        task_id="task4",
        region=REGION,
    )

    create_object >> download_object >> delete_object >> delete_batch_object

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()


from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import functools
from collections.abc import Callable
from typing import TYPE_CHECKING, Any, TypeVar

from odps import ODPS

from airflow.providers.alibaba.cloud.exceptions import MaxComputeConfigurationException
from airflow.providers.alibaba.cloud.hooks.base_alibaba import AlibabaBaseHook

if TYPE_CHECKING:
    from odps.models import Instance

RT = TypeVar("RT")


def fallback_to_default_project_endpoint(func: Callable[..., RT]) -> Callable[..., RT]:
    """
    Provide fallback for MaxCompute project and endpoint to be used as a decorator.

    If the project or endpoint is None it will be replaced with the project from the
    connection extra definition.

    :param func: function to wrap
    :return: result of the function call
    """

    @functools.wraps(func)
    def inner_wrapper(self, **kwargs) -> RT:
        required_args = ("project", "endpoint")
        for arg_name in required_args:
            # Use the value from kwargs if it is provided and value is not None, otherwise use the
            # value from the connection extra property.
            kwargs[arg_name] = getattr(self, arg_name) if kwargs.get(arg_name) is None else kwargs[arg_name]
            if not kwargs[arg_name]:
                raise MaxComputeConfigurationException(
                    f'"{arg_name}" must be passed either as '
                    "keyword parameter or as extra "
                    "in the MaxCompute connection definition. Both are not set!"
                )

        return func(self, **kwargs)

    return inner_wrapper


class MaxComputeHook(AlibabaBaseHook):
    """
    Interact with Alibaba MaxCompute (previously known as ODPS).

    :param maxcompute_conn_id: The connection ID to use when fetching connection info.
    """

    conn_name_attr = "maxcompute_conn_id"
    default_conn_name = "maxcompute_default"
    conn_type = "maxcompute"
    hook_name = "MaxCompute"

    def __init__(self, maxcompute_conn_id: str = "maxcompute_default", **kwargs) -> None:
        self.maxcompute_conn_id = maxcompute_conn_id
        super().__init__(alibabacloud_conn_id=maxcompute_conn_id, **kwargs)

    @classmethod
    def get_connection_form_widgets(cls) -> dict[str, Any]:
        """Return connection widgets to add to connection form."""
        from flask_appbuilder.fieldwidgets import BS3TextFieldWidget
        from flask_babel import lazy_gettext
        from wtforms import StringField

        connection_form_widgets = super().get_connection_form_widgets()

        connection_form_widgets["project"] = StringField(
            lazy_gettext("Project"),
            widget=BS3TextFieldWidget(),
        )
        connection_form_widgets["endpoint"] = StringField(
            lazy_gettext("Endpoint"),
            widget=BS3TextFieldWidget(),
        )

        return connection_form_widgets

    @classmethod
    def get_ui_field_behaviour(cls) -> dict[str, Any]:
        """Return custom field behaviour."""
        return {
            "hidden_fields": ["host", "schema", "login", "password", "port", "extra"],
            "relabeling": {},
        }

    @property
    def project(self) -> str:
        """
        Returns project ID.

        :return: ID of the project
        """
        return self._get_field("project")

    @property
    def endpoint(self) -> str:
        """
        Returns MaxCompute Endpoint.

        :return: Endpoint of the MaxCompute project
        """
        return self._get_field("endpoint")

    @fallback_to_default_project_endpoint
    def get_client(self, *, project: str, endpoint: str) -> ODPS:
        """
        Get an authenticated MaxCompute ODPS Client.

        :param project_id: Project ID for the project which the client acts on behalf of.
        :param location: Default location for jobs / datasets / tables.
        """
        creds = self.get_access_key_credential()

        return ODPS(
            creds.access_key_id,
            creds.access_key_secret,
            project=project,
            endpoint=endpoint,
        )

    @fallback_to_default_project_endpoint
    def run_sql(
        self,
        *,
        sql: str,
        project: str | None = None,
        endpoint: str | None = None,
        priority: int | None = None,
        running_cluster: str | None = None,
        hints: dict[str, Any] | None = None,
        aliases: dict[str, str] | None = None,
        default_schema: str | None = None,
        quota_name: str | None = None,
    ) -> Instance:
        """
        Run a given SQL statement in MaxCompute.

        The method will submit your SQL statement to MaxCompute
        and return the corresponding task Instance object.

        .. seealso:: https://pyodps.readthedocs.io/en/latest/base-sql.html#execute-sql

        :param sql: The SQL statement to run.
        :param project: The project ID to use.
        :param endpoint: The endpoint to use.
        :param priority: The priority of the SQL statement ranges from 0 to 9,
            applicable to projects with the job priority feature enabled.
            Takes precedence over the  `odps.instance.priority` setting from `hints`.
            Defaults to 9.
            See https://www.alibabacloud.com/help/en/maxcompute/user-guide/job-priority
            for details.
        :param running_cluster: The cluster to run the SQL statement on.
        :param hints: Hints for setting runtime parameters. See
            https://pyodps.readthedocs.io/en/latest/base-sql.html#id4 and
            https://www.alibabacloud.com/help/en/maxcompute/user-guide/flag-parameters
            for details.
        :param aliases: Aliases for the SQL statement.
        :param default_schema: The default schema to use.
        :param quota_name: The quota name to use.
            Defaults to project default quota if not specified.
        :return: The MaxCompute task instance.
        """
        client = self.get_client(project=project, endpoint=endpoint)

        if priority is None and hints is not None:
            priority = hints.get("odps.instance.priority")

        return client.run_sql(
            sql=sql,
            priority=priority,
            running_cluster=running_cluster,
            hints=hints,
            aliases=aliases,
            default_schema=default_schema,
            quota_name=quota_name,
        )

    @fallback_to_default_project_endpoint
    def get_instance(
        self,
        *,
        instance_id: str,
        project: str | None = None,
        endpoint: str | None = None,
    ) -> Instance:
        """
        Get a MaxCompute task instance.

        .. seealso:: https://pyodps.readthedocs.io/en/latest/base-instances.html#instances

        :param instance_id: The ID of the instance to get.
        :param project: The project ID to use.
        :param endpoint: The endpoint to use.
        :return: The MaxCompute task instance.
        :raises ValueError: If the instance does not exist.
        """
        client = self.get_client(project=project, endpoint=endpoint)

        return client.get_instance(id_=instance_id, project=project)

    @fallback_to_default_project_endpoint
    def stop_instance(
        self,
        *,
        instance_id: str,
        project: str | None = None,
        endpoint: str | None = None,
    ) -> None:
        """
        Stop a MaxCompute task instance.

        :param instance_id: The ID of the instance to stop.
        :param project: The project ID to use.
        :param endpoint: The endpoint to use.
        """
        client = self.get_client(project=project, endpoint=endpoint)

        try:
            client.stop_instance(id_=instance_id, project=project)
            self.log.info("Instance %s stop requested.", instance_id)
        except Exception:
            self.log.exception("Failed to stop instance %s.", instance_id)
            raise



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""This module contains Alibaba Cloud MaxCompute operators."""

from __future__ import annotations

from collections.abc import Sequence
from typing import TYPE_CHECKING

from airflow.providers.alibaba.cloud.hooks.maxcompute import MaxComputeHook
from airflow.providers.alibaba.cloud.links.maxcompute import MaxComputeLogViewLink
from airflow.providers.alibaba.version_compat import AIRFLOW_V_3_0_PLUS

if AIRFLOW_V_3_0_PLUS:
    from airflow.sdk import BaseOperator
else:
    from airflow.models import BaseOperator

if TYPE_CHECKING:
    from odps.models import Instance

    from airflow.utils.context import Context


class MaxComputeSQLOperator(BaseOperator):
    """
    Executes an SQL statement in MaxCompute.

    Waits for the SQL task instance to complete and returns instance id.

    :param sql: The SQL statement to run.
    :param project: The project ID to use.
    :param endpoint: The endpoint to use.
    :param priority: The priority of the SQL statement ranges from 0 to 9,
        applicable to projects with the job priority feature enabled.
        Takes precedence over the  `odps.instance.priority` setting from `hints`.
        Defaults to 9.
        See https://www.alibabacloud.com/help/en/maxcompute/user-guide/job-priority
        for details.
    :param running_cluster: The cluster to run the SQL statement on.
    :param hints: Hints for setting runtime parameters. See
        https://pyodps.readthedocs.io/en/latest/base-sql.html#id4 and
        https://www.alibabacloud.com/help/en/maxcompute/user-guide/flag-parameters
        for details.
    :param aliases: Aliases for the SQL statement.
    :param default_schema: The default schema to use.
    :param quota_name: The quota name to use.
        Defaults to project default quota if not specified.
    :param maxcompute_conn_id: The connection ID to use. Defaults to
        `maxcompute_default` if not specified.
    :param cancel_on_kill: Flag which indicates whether to stop running instance
        or not when task is killed. Default is True.
    """

    template_fields: Sequence[str] = (
        "sql",
        "project",
        "endpoint",
        "priority",
        "running_cluster",
        "hints",
        "aliases",
        "default_schema",
        "quota_name",
        "maxcompute_conn_id",
    )
    template_ext: Sequence[str] = (".sql",)
    template_fields_renderers = {"sql": "sql"}
    operator_extra_links = (MaxComputeLogViewLink(),)

    def __init__(
        self,
        *,
        sql: str,
        project: str | None = None,
        endpoint: str | None = None,
        priority: int | None = None,
        running_cluster: str | None = None,
        hints: dict[str, str] | None = None,
        aliases: dict[str, str] | None = None,
        default_schema: str | None = None,
        quota_name: str | None = None,
        maxcompute_conn_id: str = "maxcompute_default",
        cancel_on_kill: bool = True,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.sql = sql
        self.project = project
        self.endpoint = endpoint
        self.priority = priority
        self.running_cluster = running_cluster
        self.hints = hints
        self.aliases = aliases
        self.default_schema = default_schema
        self.quota_name = quota_name
        self.maxcompute_conn_id = maxcompute_conn_id
        self.cancel_on_kill = cancel_on_kill
        self.hook: MaxComputeHook | None = None
        self.instance: Instance | None = None

    def execute(self, context: Context) -> str:
        self.hook = MaxComputeHook(maxcompute_conn_id=self.maxcompute_conn_id)

        self.instance = self.hook.run_sql(
            sql=self.sql,
            project=self.project,
            endpoint=self.endpoint,
            priority=self.priority,
            running_cluster=self.running_cluster,
            hints=self.hints,
            aliases=self.aliases,
            default_schema=self.default_schema,
            quota_name=self.quota_name,
        )

        MaxComputeLogViewLink.persist(context=context, log_view_url=self.instance.get_logview_address())

        self.instance.wait_for_success()

        return self.instance.id

    def on_kill(self) -> None:
        instance_id = self.instance.id if self.instance else None

        if instance_id and self.hook and self.cancel_on_kill:
            self.hook.stop_instance(
                instance_id=instance_id,
                project=self.project,
                endpoint=self.endpoint,
            )
        else:
            self.log.info("Skipping to stop instance: %s:%s.%s", self.project, self.endpoint, instance_id)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

import pytest

from airflow.providers.alibaba.cloud.links.maxcompute import MaxComputeLogViewLink

MAXCOMPUTE_LINK_MODULE = "airflow.providers.alibaba.cloud.links.maxcompute.{}"
MOCK_TASK_ID = "run_sql"
MOCK_SQL = "SELECT 1"
MOCK_INSTANCE_ID = "mock_instance_id"


class TestMaxComputeLogViewLink:
    @pytest.mark.parametrize(
        "xcom_value, expected_link",
        [
            pytest.param("http://mock_url.com", "http://mock_url.com", id="has-log-link"),
            pytest.param(None, "", id="no-log-link"),
        ],
    )
    @mock.patch(MAXCOMPUTE_LINK_MODULE.format("XCom"))
    def test_get_link(self, mock_xcom, xcom_value, expected_link):
        mock_xcom.get_value.return_value = xcom_value

        link = MaxComputeLogViewLink().get_link(
            operator=mock.MagicMock(),
            ti_key=mock.MagicMock(),
        )

        assert link == expected_link

    def test_persist(self):
        mock_task_instance = mock.MagicMock()
        mock_context = {"task_instance": mock_task_instance}
        mock_url = "mock_url"

        MaxComputeLogViewLink.persist(
            context=mock_context,
            log_view_url=mock_url,
        )

        mock_task_instance.xcom_push.assert_called_once_with(
            key=MaxComputeLogViewLink.key,
            value=mock_url,
        )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

from airflow.providers.alibaba.cloud.hooks.oss import OSSHook

from unit.alibaba.cloud.utils.oss_mock import mock_oss_hook_default_project_id

OSS_STRING = "airflow.providers.alibaba.cloud.hooks.oss.{}"
MOCK_OSS_CONN_ID = "mock_id"
MOCK_BUCKET_NAME = "mock_bucket_name"
MOCK_KEY = "mock_key"
MOCK_KEYS = ["mock_key1", "mock_key2", "mock_key3"]
MOCK_CONTENT = "mock_content"
MOCK_FILE_PATH = "mock_file_path"


class TestOSSHook:
    def setup_method(self):
        with mock.patch(
            OSS_STRING.format("OSSHook.__init__"),
            new=mock_oss_hook_default_project_id,
        ):
            self.hook = OSSHook(oss_conn_id=MOCK_OSS_CONN_ID)

    def test_parse_oss_url(self):
        parsed = self.hook.parse_oss_url(f"oss://{MOCK_BUCKET_NAME}/this/is/not/a-real-key.txt")
        assert parsed == (MOCK_BUCKET_NAME, "this/is/not/a-real-key.txt"), "Incorrect parsing of the oss url"

    def test_parse_oss_object_directory(self):
        parsed = self.hook.parse_oss_url(f"oss://{MOCK_BUCKET_NAME}/this/is/not/a-real-oss-directory/")
        assert parsed == (
            MOCK_BUCKET_NAME,
            "this/is/not/a-real-oss-directory/",
        ), "Incorrect parsing of the oss url"

    @mock.patch(OSS_STRING.format("oss2"))
    def test_get_credential(self, mock_oss2):
        self.hook.get_credential()
        mock_oss2.Auth.assert_called_once_with("mock_access_key_id", "mock_access_key_secret")

    @mock.patch(OSS_STRING.format("OSSHook.get_credential"))
    @mock.patch(OSS_STRING.format("oss2"))
    def test_get_bucket(self, mock_oss2, mock_get_credential):
        self.hook.get_bucket("mock_bucket_name")
        mock_get_credential.assert_called_once_with()
        mock_oss2.Bucket.assert_called_once_with(
            mock_get_credential.return_value, "https://oss-mock_region.aliyuncs.com", MOCK_BUCKET_NAME
        )

    @mock.patch(OSS_STRING.format("OSSHook.get_bucket"))
    def test_object_exist(self, mock_service):
        # Given
        mock_bucket = mock_service.return_value
        exists_method = mock_bucket.object_exists
        exists_method.return_value = True

        # When
        res = self.hook.object_exists(MOCK_KEY, MOCK_BUCKET_NAME)

        # Then
        assert res is True
        mock_service.assert_called_once_with(MOCK_BUCKET_NAME)
        exists_method.assert_called_once_with(MOCK_KEY)

    @mock.patch(OSS_STRING.format("OSSHook.get_bucket"))
    def test_load_string(self, mock_service):
        self.hook.load_string(MOCK_KEY, MOCK_CONTENT, MOCK_BUCKET_NAME)
        mock_service.assert_called_once_with(MOCK_BUCKET_NAME)
        mock_service.return_value.put_object.assert_called_once_with(MOCK_KEY, MOCK_CONTENT)

    @mock.patch(OSS_STRING.format("OSSHook.get_bucket"))
    def test_upload_local_file(self, mock_service):
        self.hook.upload_local_file(MOCK_KEY, MOCK_FILE_PATH, MOCK_BUCKET_NAME)
        mock_service.assert_called_once_with(MOCK_BUCKET_NAME)
        mock_service.return_value.put_object_from_file.assert_called_once_with(MOCK_KEY, MOCK_FILE_PATH)

    @mock.patch(OSS_STRING.format("OSSHook.get_bucket"))
    def test_download_file(self, mock_service):
        self.hook.download_file(MOCK_KEY, MOCK_FILE_PATH, MOCK_BUCKET_NAME)
        mock_service.assert_called_once_with(MOCK_BUCKET_NAME)
        mock_service.return_value.get_object_to_file.assert_called_once_with(MOCK_KEY, MOCK_FILE_PATH)

    @mock.patch(OSS_STRING.format("OSSHook.get_bucket"))
    def test_delete_object(self, mock_service):
        self.hook.delete_object(MOCK_KEY, MOCK_BUCKET_NAME)
        mock_service.assert_called_once_with(MOCK_BUCKET_NAME)
        mock_service.return_value.delete_object.assert_called_once_with(MOCK_KEY)

    @mock.patch(OSS_STRING.format("OSSHook.get_bucket"))
    def test_delete_objects(self, mock_service):
        self.hook.delete_objects(MOCK_KEYS, MOCK_BUCKET_NAME)
        mock_service.assert_called_once_with(MOCK_BUCKET_NAME)
        mock_service.return_value.batch_delete_objects.assert_called_once_with(MOCK_KEYS)

    @mock.patch(OSS_STRING.format("OSSHook.get_bucket"))
    def test_delete_bucket(self, mock_service):
        self.hook.delete_bucket(MOCK_BUCKET_NAME)
        mock_service.assert_called_once_with(MOCK_BUCKET_NAME)
        mock_service.return_value.delete_bucket.assert_called_once_with()

    @mock.patch(OSS_STRING.format("OSSHook.get_bucket"))
    def test_create_bucket(self, mock_service):
        self.hook.create_bucket(MOCK_BUCKET_NAME)
        mock_service.assert_called_once_with(MOCK_BUCKET_NAME)
        mock_service.return_value.create_bucket.assert_called_once_with()

    @mock.patch(OSS_STRING.format("OSSHook.get_bucket"))
    def test_append_string(self, mock_service):
        self.hook.append_string(MOCK_BUCKET_NAME, MOCK_CONTENT, MOCK_KEY, 0)
        mock_service.assert_called_once_with(MOCK_BUCKET_NAME)
        mock_service.return_value.append_object.assert_called_once_with(MOCK_KEY, 0, MOCK_CONTENT)

    @mock.patch(OSS_STRING.format("OSSHook.get_bucket"))
    def test_read_key(self, mock_service):
        # Given
        mock_service.return_value.get_object.return_value.read.return_value.decode.return_value = MOCK_CONTENT

        # When
        res = self.hook.read_key(MOCK_BUCKET_NAME, MOCK_KEY)

        # Then
        assert res == MOCK_CONTENT
        mock_service.assert_called_once_with(MOCK_BUCKET_NAME)
        mock_service.return_value.get_object.assert_called_once_with(MOCK_KEY)
        mock_service.return_value.get_object.return_value.read.assert_called_once_with()
        mock_service.return_value.get_object.return_value.read.return_value.decode.assert_called_once_with(
            "utf-8"
        )

    @mock.patch(OSS_STRING.format("OSSHook.get_bucket"))
    def test_head_key(self, mock_service):
        self.hook.head_key(MOCK_BUCKET_NAME, MOCK_KEY)
        mock_service.assert_called_once_with(MOCK_BUCKET_NAME)
        mock_service.return_value.head_object.assert_called_once_with(MOCK_KEY)

    @mock.patch(OSS_STRING.format("OSSHook.get_bucket"))
    def test_key_exists(self, mock_service):
        # When
        mock_service.return_value.object_exists.return_value = True

        # Given
        res = self.hook.key_exist(MOCK_BUCKET_NAME, MOCK_KEY)

        # Then
        assert res is True
        mock_service.assert_called_once_with(MOCK_BUCKET_NAME)
        mock_service.return_value.object_exists.assert_called_once_with(MOCK_KEY)

    def test_get_default_region(self):
        assert self.hook.get_default_region() == "mock_region"



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

from airflow.providers.alibaba.cloud.hooks.base_alibaba import AccessKeyCredentials, AlibabaBaseHook

BASE_ALIBABA_HOOK_MODULE = "airflow.providers.alibaba.cloud.hooks.base_alibaba.{}"
MOCK_MAXCOMPUTE_CONN_ID = "mock_id"
MOCK_ACCESS_KEY_ID = "mock_access_key_id"
MOCK_ACCESS_KEY_SECRET = "mock_access_key_secret"


class TestAlibabaBaseHook:
    def setup_method(self):
        with mock.patch(
            BASE_ALIBABA_HOOK_MODULE.format("AlibabaBaseHook.get_connection"),
        ) as mock_get_connection:
            mock_conn = mock.MagicMock()
            mock_conn.extra_dejson = {
                "access_key_id": MOCK_ACCESS_KEY_ID,
                "access_key_secret": MOCK_ACCESS_KEY_SECRET,
            }

            mock_get_connection.return_value = mock_conn
            self.hook = AlibabaBaseHook(alibabacloud_conn_id=MOCK_MAXCOMPUTE_CONN_ID)

    def test_get_access_key_credential(self):
        creds = AccessKeyCredentials(
            access_key_id=MOCK_ACCESS_KEY_ID,
            access_key_secret=MOCK_ACCESS_KEY_SECRET,
        )

        creds = self.hook.get_access_key_credential()

        assert creds.access_key_id == MOCK_ACCESS_KEY_ID
        assert creds.access_key_secret == MOCK_ACCESS_KEY_SECRET



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

from airflow.providers.alibaba.cloud.sensors.analyticdb_spark import AnalyticDBSparkSensor
from airflow.utils import timezone

ADB_SPARK_SENSOR_STRING = "airflow.providers.alibaba.cloud.sensors.analyticdb_spark.{}"
DEFAULT_DATE = timezone.datetime(2017, 1, 1)
MOCK_ADB_SPARK_CONN_ID = "mock_adb_spark_default"
MOCK_ADB_SPARK_ID = "mock_adb_spark_id"
MOCK_SENSOR_TASK_ID = "test-adb-spark-operator"
MOCK_REGION = "mock_region"


class TestAnalyticDBSparkSensor:
    def setup_method(self):
        self.sensor = AnalyticDBSparkSensor(
            app_id=MOCK_ADB_SPARK_ID,
            adb_spark_conn_id=MOCK_ADB_SPARK_CONN_ID,
            region=MOCK_REGION,
            task_id=MOCK_SENSOR_TASK_ID,
        )

    @mock.patch(ADB_SPARK_SENSOR_STRING.format("AnalyticDBSparkHook"))
    def test_get_hook(self, mock_service):
        """Test get_hook function works as expected."""
        self.sensor.hook
        mock_service.assert_called_once_with(adb_spark_conn_id=MOCK_ADB_SPARK_CONN_ID, region=MOCK_REGION)

    @mock.patch(ADB_SPARK_SENSOR_STRING.format("AnalyticDBSparkSensor.hook"))
    def test_poke_terminal_state(self, mock_service):
        """Test poke_terminal_state works as expected with COMPLETED application."""
        # Given
        mock_service.get_spark_state.return_value = "COMPLETED"

        # When
        res = self.sensor.poke(None)

        # Then
        assert res is True
        mock_service.get_spark_state.assert_called_once_with(MOCK_ADB_SPARK_ID)

    @mock.patch(ADB_SPARK_SENSOR_STRING.format("AnalyticDBSparkSensor.hook"))
    def test_poke_non_terminal_state(self, mock_service):
        """Test poke_terminal_state works as expected with RUNNING application."""
        # Given
        mock_service.get_spark_state.return_value = "RUNNING"

        # When
        res = self.sensor.poke(None)

        # Then
        assert res is False
        mock_service.get_spark_state.assert_called_once_with(MOCK_ADB_SPARK_ID)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from collections.abc import Callable
from functools import wraps
from inspect import signature
from typing import TYPE_CHECKING, TypeVar, cast
from urllib.parse import urlsplit

import oss2
from oss2.exceptions import ClientError

from airflow.exceptions import AirflowException

try:
    from airflow.sdk import BaseHook
except ImportError:
    from airflow.hooks.base import BaseHook  # type: ignore[attr-defined,no-redef]

if TYPE_CHECKING:
    try:
        from airflow.sdk import Connection
    except ImportError:
        from airflow.models.connection import Connection  # type: ignore[assignment]

T = TypeVar("T", bound=Callable)


def provide_bucket_name(func: T) -> T:
    """Unify bucket name and key if a key is provided but not a bucket name."""
    function_signature = signature(func)

    @wraps(func)
    def wrapper(*args, **kwargs) -> T:
        bound_args = function_signature.bind(*args, **kwargs)
        self = args[0]
        if bound_args.arguments.get("bucket_name") is None and self.oss_conn_id:
            connection = self.get_connection(self.oss_conn_id)
            if connection.schema:
                bound_args.arguments["bucket_name"] = connection.schema

        return func(*bound_args.args, **bound_args.kwargs)

    return cast("T", wrapper)


def unify_bucket_name_and_key(func: T) -> T:
    """Unify bucket name and key if a key is provided but not a bucket name."""
    function_signature = signature(func)

    @wraps(func)
    def wrapper(*args, **kwargs) -> T:
        bound_args = function_signature.bind(*args, **kwargs)

        def get_key() -> str:
            if "key" in bound_args.arguments:
                return "key"
            raise ValueError("Missing key parameter!")

        key_name = get_key()
        if bound_args.arguments.get("bucket_name") is None:
            bound_args.arguments["bucket_name"], bound_args.arguments["key"] = OSSHook.parse_oss_url(
                bound_args.arguments[key_name]
            )

        return func(*bound_args.args, **bound_args.kwargs)

    return cast("T", wrapper)


class OSSHook(BaseHook):
    """Interact with Alibaba Cloud OSS, using the oss2 library."""

    conn_name_attr = "oss_conn_id"
    default_conn_name = "oss_default"
    conn_type = "oss"
    hook_name = "OSS"

    def __init__(self, region: str | None = None, oss_conn_id="oss_default", *args, **kwargs) -> None:
        self.oss_conn_id = oss_conn_id
        self.oss_conn = self.get_connection(oss_conn_id)
        self.region = region or self.get_default_region()
        super().__init__(*args, **kwargs)

    def get_conn(self) -> Connection:
        """Return connection for the hook."""
        return self.oss_conn

    @staticmethod
    def parse_oss_url(ossurl: str) -> tuple:
        """
        Parse the OSS Url into a bucket name and key.

        :param ossurl: The OSS Url to parse.
        :return: the parsed bucket name and key
        """
        parsed_url = urlsplit(ossurl)

        if not parsed_url.netloc:
            raise AirflowException(f'Please provide a bucket_name instead of "{ossurl}"')

        bucket_name = parsed_url.netloc
        key = parsed_url.path.lstrip("/")

        return bucket_name, key

    @provide_bucket_name
    @unify_bucket_name_and_key
    def object_exists(self, key: str, bucket_name: str | None = None) -> bool:
        """
        Check if object exists.

        :param key: the path of the object
        :param bucket_name: the name of the bucket
        :return: True if it exists and False if not.
        """
        try:
            return self.get_bucket(bucket_name).object_exists(key)
        except ClientError as e:
            self.log.error(e.message)
            return False

    @provide_bucket_name
    def get_bucket(self, bucket_name: str | None = None) -> oss2.api.Bucket:
        """
        Return a oss2.Bucket object.

        :param bucket_name: the name of the bucket
        :return: the bucket object to the bucket name.
        """
        auth = self.get_credential()
        return oss2.Bucket(auth, f"https://oss-{self.region}.aliyuncs.com", bucket_name)

    @provide_bucket_name
    @unify_bucket_name_and_key
    def load_string(self, key: str, content: str, bucket_name: str | None = None) -> None:
        """
        Load a string to OSS.

        :param key: the path of the object
        :param content: str to set as content for the key.
        :param bucket_name: the name of the bucket
        """
        try:
            self.get_bucket(bucket_name).put_object(key, content)
        except Exception as e:
            raise AirflowException(f"Errors: {e}")

    @provide_bucket_name
    @unify_bucket_name_and_key
    def upload_local_file(
        self,
        key: str,
        file: str,
        bucket_name: str | None = None,
    ) -> None:
        """
        Upload a local file to OSS.

        :param key: the OSS path of the object
        :param file: local file to upload.
        :param bucket_name: the name of the bucket
        """
        try:
            self.get_bucket(bucket_name).put_object_from_file(key, file)
        except Exception as e:
            raise AirflowException(f"Errors when upload file: {e}")

    @provide_bucket_name
    @unify_bucket_name_and_key
    def download_file(
        self,
        key: str,
        local_file: str,
        bucket_name: str | None = None,
    ) -> str | None:
        """
        Download file from OSS.

        :param key: key of the file-like object to download.
        :param local_file: local path + file name to save.
        :param bucket_name: the name of the bucket
        :return: the file name.
        """
        try:
            self.get_bucket(bucket_name).get_object_to_file(key, local_file)
        except Exception as e:
            self.log.error(e)
            return None
        return local_file

    @provide_bucket_name
    @unify_bucket_name_and_key
    def delete_object(
        self,
        key: str,
        bucket_name: str | None = None,
    ) -> None:
        """
        Delete object from OSS.

        :param key: key of the object to delete.
        :param bucket_name: the name of the bucket
        """
        try:
            self.get_bucket(bucket_name).delete_object(key)
        except Exception as e:
            self.log.error(e)
            raise AirflowException(f"Errors when deleting: {key}")

    @provide_bucket_name
    @unify_bucket_name_and_key
    def delete_objects(
        self,
        key: list,
        bucket_name: str | None = None,
    ) -> None:
        """
        Delete objects from OSS.

        :param key: keys list of the objects to delete.
        :param bucket_name: the name of the bucket
        """
        try:
            self.get_bucket(bucket_name).batch_delete_objects(key)
        except Exception as e:
            self.log.error(e)
            raise AirflowException(f"Errors when deleting: {key}")

    @provide_bucket_name
    def delete_bucket(
        self,
        bucket_name: str | None = None,
    ) -> None:
        """
        Delete bucket from OSS.

        :param bucket_name: the name of the bucket
        """
        try:
            self.get_bucket(bucket_name).delete_bucket()
        except Exception as e:
            self.log.error(e)
            raise AirflowException(f"Errors when deleting: {bucket_name}")

    @provide_bucket_name
    def create_bucket(
        self,
        bucket_name: str | None = None,
    ) -> None:
        """
        Create bucket.

        :param bucket_name: the name of the bucket
        """
        try:
            self.get_bucket(bucket_name).create_bucket()
        except Exception as e:
            self.log.error(e)
            raise AirflowException(f"Errors when create bucket: {bucket_name}")

    @provide_bucket_name
    @unify_bucket_name_and_key
    def append_string(self, bucket_name: str | None, content: str, key: str, pos: int) -> None:
        """
        Append string to a remote existing file.

        :param bucket_name: the name of the bucket
        :param content: content to be appended
        :param key: oss bucket key
        :param pos: position of the existing file where the content will be appended
        """
        self.log.info("Write oss bucket. key: %s, pos: %s", key, pos)
        try:
            self.get_bucket(bucket_name).append_object(key, pos, content)
        except Exception as e:
            self.log.error(e)
            raise AirflowException(f"Errors when append string for object: {key}")

    @provide_bucket_name
    @unify_bucket_name_and_key
    def read_key(self, bucket_name: str | None, key: str) -> str:
        """
        Read oss remote object content with the specified key.

        :param bucket_name: the name of the bucket
        :param key: oss bucket key
        """
        self.log.info("Read oss key: %s", key)
        try:
            return self.get_bucket(bucket_name).get_object(key).read().decode("utf-8")
        except Exception as e:
            self.log.error(e)
            raise AirflowException(f"Errors when read bucket object: {key}")

    @provide_bucket_name
    @unify_bucket_name_and_key
    def head_key(self, bucket_name: str | None, key: str) -> oss2.models.HeadObjectResult:
        """
        Get meta info of the specified remote object.

        :param bucket_name: the name of the bucket
        :param key: oss bucket key
        """
        self.log.info("Head Object oss key: %s", key)
        try:
            return self.get_bucket(bucket_name).head_object(key)
        except Exception as e:
            self.log.error(e)
            raise AirflowException(f"Errors when head bucket object: {key}")

    @provide_bucket_name
    @unify_bucket_name_and_key
    def key_exist(self, bucket_name: str | None, key: str) -> bool:
        """
        Find out whether the specified key exists in the oss remote storage.

        :param bucket_name: the name of the bucket
        :param key: oss bucket key
        """
        # full_path = None
        self.log.info("Looking up oss bucket %s for bucket key %s ...", bucket_name, key)
        try:
            return self.get_bucket(bucket_name).object_exists(key)
        except Exception as e:
            self.log.error(e)
            raise AirflowException(f"Errors when check bucket object existence: {key}")

    def get_credential(self) -> oss2.auth.Auth:
        extra_config = self.oss_conn.extra_dejson
        auth_type = extra_config.get("auth_type", None)
        if not auth_type:
            raise ValueError("No auth_type specified in extra_config. ")

        if auth_type != "AK":
            raise ValueError(f"Unsupported auth_type: {auth_type}")
        oss_access_key_id = extra_config.get("access_key_id", None)
        oss_access_key_secret = extra_config.get("access_key_secret", None)
        if not oss_access_key_id:
            raise ValueError(f"No access_key_id is specified for connection: {self.oss_conn_id}")

        if not oss_access_key_secret:
            raise ValueError(f"No access_key_secret is specified for connection: {self.oss_conn_id}")

        return oss2.Auth(oss_access_key_id, oss_access_key_secret)

    def get_default_region(self) -> str:
        extra_config = self.oss_conn.extra_dejson
        auth_type = extra_config.get("auth_type", None)
        if not auth_type:
            raise ValueError("No auth_type specified in extra_config. ")

        if auth_type != "AK":
            raise ValueError(f"Unsupported auth_type: {auth_type}")

        default_region = extra_config.get("region", None)
        if not default_region:
            raise ValueError(f"No region is specified for connection: {self.oss_conn_id}")
        return default_region



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

from airflow.providers.alibaba.cloud.operators.oss import (
    OSSCreateBucketOperator,
    OSSDeleteBatchObjectOperator,
    OSSDeleteBucketOperator,
    OSSDeleteObjectOperator,
    OSSDownloadObjectOperator,
    OSSUploadObjectOperator,
)

MOCK_TASK_ID = "test-oss-operator"
MOCK_REGION = "mock_region"
MOCK_BUCKET = "mock_bucket_name"
MOCK_OSS_CONN_ID = "mock_oss_conn_default"
MOCK_KEY = "mock_key"
MOCK_KEYS = ["mock_key1", "mock_key_2", "mock_key3"]
MOCK_CONTENT = "mock_content"


class TestOSSCreateBucketOperator:
    @mock.patch("airflow.providers.alibaba.cloud.operators.oss.OSSHook")
    def test_execute(self, mock_hook):
        operator = OSSCreateBucketOperator(
            task_id=MOCK_TASK_ID, region=MOCK_REGION, bucket_name=MOCK_BUCKET, oss_conn_id=MOCK_OSS_CONN_ID
        )
        operator.execute(None)
        mock_hook.assert_called_once_with(oss_conn_id=MOCK_OSS_CONN_ID, region=MOCK_REGION)
        mock_hook.return_value.create_bucket.assert_called_once_with(bucket_name=MOCK_BUCKET)


class TestOSSDeleteBucketOperator:
    @mock.patch("airflow.providers.alibaba.cloud.operators.oss.OSSHook")
    def test_execute(self, mock_hook):
        operator = OSSDeleteBucketOperator(
            task_id=MOCK_TASK_ID, region=MOCK_REGION, bucket_name=MOCK_BUCKET, oss_conn_id=MOCK_OSS_CONN_ID
        )
        operator.execute(None)
        mock_hook.assert_called_once_with(oss_conn_id=MOCK_OSS_CONN_ID, region=MOCK_REGION)
        mock_hook.return_value.delete_bucket.assert_called_once_with(bucket_name=MOCK_BUCKET)


class TestOSSUploadObjectOperator:
    @mock.patch("airflow.providers.alibaba.cloud.operators.oss.OSSHook")
    def test_execute(self, mock_hook):
        operator = OSSUploadObjectOperator(
            task_id=MOCK_TASK_ID,
            region=MOCK_REGION,
            bucket_name=MOCK_BUCKET,
            oss_conn_id=MOCK_OSS_CONN_ID,
            key=MOCK_KEY,
            file=MOCK_CONTENT,
        )
        operator.execute(None)
        mock_hook.assert_called_once_with(oss_conn_id=MOCK_OSS_CONN_ID, region=MOCK_REGION)
        mock_hook.return_value.upload_local_file.assert_called_once_with(
            bucket_name=MOCK_BUCKET, key=MOCK_KEY, file=MOCK_CONTENT
        )


class TestOSSDownloadObjectOperator:
    @mock.patch("airflow.providers.alibaba.cloud.operators.oss.OSSHook")
    def test_execute(self, mock_hook):
        operator = OSSDownloadObjectOperator(
            task_id=MOCK_TASK_ID,
            region=MOCK_REGION,
            bucket_name=MOCK_BUCKET,
            oss_conn_id=MOCK_OSS_CONN_ID,
            key=MOCK_KEY,
            file=MOCK_CONTENT,
        )
        operator.execute(None)
        mock_hook.assert_called_once_with(oss_conn_id=MOCK_OSS_CONN_ID, region=MOCK_REGION)
        mock_hook.return_value.download_file.assert_called_once_with(
            bucket_name=MOCK_BUCKET, key=MOCK_KEY, local_file=MOCK_CONTENT
        )


class TestOSSDeleteBatchObjectOperator:
    @mock.patch("airflow.providers.alibaba.cloud.operators.oss.OSSHook")
    def test_execute(self, mock_hook):
        operator = OSSDeleteBatchObjectOperator(
            task_id=MOCK_TASK_ID,
            region=MOCK_REGION,
            bucket_name=MOCK_BUCKET,
            oss_conn_id=MOCK_OSS_CONN_ID,
            keys=MOCK_KEYS,
        )
        operator.execute(None)
        mock_hook.assert_called_once_with(oss_conn_id=MOCK_OSS_CONN_ID, region=MOCK_REGION)
        mock_hook.return_value.delete_objects.assert_called_once_with(bucket_name=MOCK_BUCKET, key=MOCK_KEYS)


class TestOSSDeleteObjectOperator:
    @mock.patch("airflow.providers.alibaba.cloud.operators.oss.OSSHook")
    def test_execute(self, mock_hook):
        operator = OSSDeleteObjectOperator(
            task_id=MOCK_TASK_ID,
            region=MOCK_REGION,
            bucket_name=MOCK_BUCKET,
            oss_conn_id=MOCK_OSS_CONN_ID,
            key=MOCK_KEY,
        )
        operator.execute(None)
        mock_hook.assert_called_once_with(oss_conn_id=MOCK_OSS_CONN_ID, region=MOCK_REGION)
        mock_hook.return_value.delete_object.assert_called_once_with(bucket_name=MOCK_BUCKET, key=MOCK_KEY)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json

from airflow.models import Connection

OSS_PROJECT_ID_HOOK_UNIT_TEST = "example-project"


def mock_oss_hook_default_project_id(self, oss_conn_id="mock_oss_default", region="mock_region"):
    self.oss_conn_id = oss_conn_id
    self.oss_conn = Connection(
        extra=json.dumps(
            {
                "auth_type": "AK",
                "access_key_id": "mock_access_key_id",
                "access_key_secret": "mock_access_key_secret",
                "region": "mock_region",
            }
        )
    )
    self.region = region



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

from alibabacloud_adb20211201.models import (
    GetSparkAppLogResponse,
    GetSparkAppLogResponseBody,
    GetSparkAppLogResponseBodyData,
    GetSparkAppStateResponse,
    GetSparkAppStateResponseBody,
    GetSparkAppStateResponseBodyData,
    GetSparkAppWebUiAddressResponse,
    GetSparkAppWebUiAddressResponseBody,
    GetSparkAppWebUiAddressResponseBodyData,
    KillSparkAppResponse,
    SubmitSparkAppResponse,
)

from airflow.providers.alibaba.cloud.hooks.analyticdb_spark import AnalyticDBSparkHook

from unit.alibaba.cloud.utils.analyticdb_spark_mock import mock_adb_spark_hook_default_project_id

ADB_SPARK_STRING = "airflow.providers.alibaba.cloud.hooks.analyticdb_spark.{}"
MOCK_ADB_SPARK_CONN_ID = "mock_id"
MOCK_ADB_CLUSTER_ID = "mock_adb_cluster_id"
MOCK_ADB_RG_NAME = "mock_adb_rg_name"
MOCK_ADB_SPARK_ID = "mock_adb_spark_id"


class TestAnalyticDBSparkHook:
    def setup_method(self):
        with mock.patch(
            ADB_SPARK_STRING.format("AnalyticDBSparkHook.__init__"),
            new=mock_adb_spark_hook_default_project_id,
        ):
            self.hook = AnalyticDBSparkHook(adb_spark_conn_id=MOCK_ADB_SPARK_CONN_ID)

    def test_build_submit_app_data(self):
        """Test build submit application data for analyticDB spark as expected."""
        res_data = self.hook.build_submit_app_data(
            file="oss://test_file",
            class_name="com.aliyun.spark.SparkPi",
            args=[1000, "test-args"],
            conf={"spark.executor.instances": 1, "spark.eventLog.enabled": "true"},
            jars=["oss://1.jar", "oss://2.jar"],
            py_files=["oss://1.py", "oss://2.py"],
            files=["oss://1.file", "oss://2.file"],
            driver_resource_spec="medium",
            executor_resource_spec="medium",
            num_executors=2,
            archives=["oss://1.zip", "oss://2.zip"],
            name="test",
        )
        except_data = {
            "file": "oss://test_file",
            "className": "com.aliyun.spark.SparkPi",
            "args": ["1000", "test-args"],
            "conf": {
                "spark.executor.instances": 1,
                "spark.eventLog.enabled": "true",
                "spark.driver.resourceSpec": "medium",
                "spark.executor.resourceSpec": "medium",
            },
            "jars": ["oss://1.jar", "oss://2.jar"],
            "pyFiles": ["oss://1.py", "oss://2.py"],
            "files": ["oss://1.file", "oss://2.file"],
            "archives": ["oss://1.zip", "oss://2.zip"],
            "name": "test",
        }
        assert res_data == except_data

    def test_build_submit_sql_data(self):
        """Test build submit sql data for analyticDB spark as expected."""
        res_data = self.hook.build_submit_sql_data(
            sql="""
            set spark.executor.instances=1;
            show databases;
            """,
            conf={"spark.executor.instances": 2},
            driver_resource_spec="medium",
            executor_resource_spec="medium",
            num_executors=3,
            name="test",
        )
        except_data = (
            "set spark.driver.resourceSpec = medium;set spark.executor.resourceSpec = medium;set "
            "spark.executor.instances = 2;set spark.app.name = test;\n            set "
            "spark.executor.instances=1;\n            show databases;"
        )
        assert res_data == except_data

    @mock.patch(ADB_SPARK_STRING.format("AnalyticDBSparkHook.get_adb_spark_client"))
    def test_submit_spark_app(self, mock_service):
        """Test submit_spark_app function works as expected."""
        # Given
        mock_client = mock_service.return_value
        exists_method = mock_client.submit_spark_app
        exists_method.return_value = SubmitSparkAppResponse(status_code=200)

        # When
        res = self.hook.submit_spark_app(MOCK_ADB_CLUSTER_ID, MOCK_ADB_RG_NAME, "oss://test.py")

        # Then
        assert isinstance(res, SubmitSparkAppResponse)
        mock_service.assert_called_once_with()

    @mock.patch(ADB_SPARK_STRING.format("AnalyticDBSparkHook.get_adb_spark_client"))
    def test_submit_spark_sql(self, mock_service):
        """Test submit_spark_app function works as expected."""
        # Given
        mock_client = mock_service.return_value
        exists_method = mock_client.submit_spark_app
        exists_method.return_value = SubmitSparkAppResponse(status_code=200)

        # When
        res = self.hook.submit_spark_sql(MOCK_ADB_CLUSTER_ID, MOCK_ADB_RG_NAME, "SELECT 1")

        # Then
        assert isinstance(res, SubmitSparkAppResponse)
        mock_service.assert_called_once_with()

    @mock.patch(ADB_SPARK_STRING.format("AnalyticDBSparkHook.get_adb_spark_client"))
    def test_get_spark_state(self, mock_service):
        """Test get_spark_state function works as expected."""
        # Given
        mock_client = mock_service.return_value
        exists_method = mock_client.get_spark_app_state
        exists_method.return_value = GetSparkAppStateResponse(
            body=GetSparkAppStateResponseBody(data=GetSparkAppStateResponseBodyData(state="RUNNING"))
        )

        # When
        res = self.hook.get_spark_state(MOCK_ADB_SPARK_ID)

        # Then
        assert res == "RUNNING"
        mock_service.assert_called_once_with()

    @mock.patch(ADB_SPARK_STRING.format("AnalyticDBSparkHook.get_adb_spark_client"))
    def test_get_spark_web_ui_address(self, mock_service):
        """Test get_spark_web_ui_address function works as expected."""
        # Given
        mock_client = mock_service.return_value
        exists_method = mock_client.get_spark_app_web_ui_address
        exists_method.return_value = GetSparkAppWebUiAddressResponse(
            body=GetSparkAppWebUiAddressResponseBody(
                data=GetSparkAppWebUiAddressResponseBodyData(web_ui_address="https://mock-web-ui-address.com")
            )
        )

        # When
        res = self.hook.get_spark_web_ui_address(MOCK_ADB_SPARK_ID)

        # Then
        assert res == "https://mock-web-ui-address.com"
        mock_service.assert_called_once_with()

    @mock.patch(ADB_SPARK_STRING.format("AnalyticDBSparkHook.get_adb_spark_client"))
    def test_get_spark_log(self, mock_service):
        """Test get_spark_log function works as expected."""
        # Given
        mock_client = mock_service.return_value
        exists_method = mock_client.get_spark_app_log
        exists_method.return_value = GetSparkAppLogResponse(
            body=GetSparkAppLogResponseBody(data=GetSparkAppLogResponseBodyData(log_content="Pi is 3.14"))
        )

        # When
        res = self.hook.get_spark_log(MOCK_ADB_SPARK_ID)

        # Then
        assert res == "Pi is 3.14"
        mock_service.assert_called_once_with()

    @mock.patch(ADB_SPARK_STRING.format("AnalyticDBSparkHook.get_adb_spark_client"))
    def test_kill_spark_app(self, mock_service):
        """Test kill_spark_app function works as expected."""
        # Given
        mock_client = mock_service.return_value
        exists_method = mock_client.kill_spark_app
        exists_method.return_value = KillSparkAppResponse()

        # When
        self.hook.kill_spark_app(MOCK_ADB_SPARK_ID)

        # Then
        mock_service.assert_called_once_with()



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import os
from datetime import datetime

from airflow.models.dag import DAG
from airflow.providers.alibaba.cloud.operators.analyticdb_spark import AnalyticDBSparkSQLOperator

# Ignore missing args provided by default_args
# mypy: disable-error-code="call-arg"

ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID")
DAG_ID = "adb_spark_sql_dag"
# [START howto_operator_adb_spark_sql]
with DAG(
    dag_id=DAG_ID,
    start_date=datetime(2021, 1, 1),
    schedule=None,
    default_args={"cluster_id": "your cluster", "rg_name": "your resource group", "region": "your region"},
    max_active_runs=1,
    catchup=False,
) as dag:
    show_databases = AnalyticDBSparkSQLOperator(
        task_id="task1",
        sql="SHOE DATABASES;",
        cluster_id="<your cluster id>",
        rg_name="<your resource group name>",
    )

    show_tables = AnalyticDBSparkSQLOperator(
        task_id="task2",
        sql="SHOW TABLES;",
        cluster_id="<your cluster id>",
        rg_name="<your resource group name>",
    )

    #  Replace the above cluster_id and rg_name with your own values.

    show_databases >> show_tables

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()
# [END howto_operator_adb_spark_sql]

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)
